/**
 * @author Ilias Gazdaliev <invimind@gmail.com>
 */
import 'reflect-metadata';
/**
 * JsonProperty params
 *
 * @param {string} path -- path pattern for the property
 * supports every pattern provided by lodash/get|set object
 * @param {boolean} required throws an Error if json is missing required property
 * @param {Function} type Optional. In most cases there is no need to specify
 * type explicitly
 * @param {Function} elementType due to reflect-metadata restriction one should
 * explicitly set elementType property to correctly serialize/deserialize Array
 * or Set values
 * @param {Function} validate function for validating json values. Throws an error
 * if property fails to pass validate check
 * @param {Function} deserialize function for custom deserialization
 * @param {Function} serialize function for custom serialization
 * @param {Function} afterDeserialize takes deserialized instance and deserialized property. Should return new property value.
 */
declare type Params<P> = {
    path?: string;
    required?: boolean;
    type?: new (...params: Array<unknown>) => unknown;
    elementType?: new (...args: any[]) => unknown;
    validate?: (property: P) => boolean;
    deserialize?: (jsonValue: unknown) => P;
    serialize?: (property: P) => unknown;
    afterDeserialize?: (deserializedInstance: InstanceType<new (...args: any[]) => unknown>, propertyValue: P) => P;
};
export declare type JsonPropertyMetadata<P = unknown> = {
    name: string;
    path: string;
    type: new (...params: any[]) => unknown;
} & Params<P>;
/**
 * Decorator. Collects annotated property metadata.
 * @param {string | Params} arg
 */
export default function JsonProperty<P = unknown>(arg?: Params<P> | string): (object: Object, propertyName: string) => void;
export {};
